/*
 * ============================================================================
 *                   The Apache Software License, Version 1.1
 * ============================================================================
 * 
 *    Copyright (C) 1999 The Apache Software Foundation. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include  the following  acknowledgment:  "This product includes  software
 *    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
 *    Alternately, this  acknowledgment may  appear in the software itself,  if
 *    and wherever such third-party acknowledgments normally appear.
 * 
 * 4. The names "Ant" and  "Apache Software Foundation"  must not be used to
 *    endorse  or promote  products derived  from this  software without  prior
 *    written permission. For written permission, please contact
 *    apache@apache.org.
 * 
 * 5. Products  derived from this software may not  be called "Apache", nor may
 *    "Apache" appear  in their name,  without prior written permission  of the
 *    Apache Software Foundation.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 * DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the  Apache Software Foundation.  For more  information  on the 
 * Apache Software Foundation, please see <http://www.apache.org/>.
 *
 */

package org.apache.tools.ant.taskdefs.optional.javarec;

import java.io.File;
import java.util.Vector;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.Execute;
import org.apache.tools.ant.types.Commandline;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Reference;

/**
 * Generate a record class from a record type class.
 * This Ant task calls the <code>Generator</code> command to
 * create <code>Record</code> classes from <code>RecordType</code>
 * classes.  There are no nested tags for this task, only
 * attributes.
 * <h3>Parameters</h3>
 * <table border="1" cellpadding="2">
 * <tr><th>Attribute<th>Description<th>Required
 * <tr><td><tt>targetDir</tt>
 *     <td>The directory into which record source will be generated.
 *     <td>Yes
 * <tr><td><tt>pkgName</tt>
 *     <td>The package in which to place the record class.
 *     <td>Yes
 * <tr><td><tt>className</tt>
 *     <td>The name of the record class.
 *     <td>Yes
 * <tr><td><tt>outputType</tt>
 *     <td><ul>
 *         <li><b>0</b> - Custom Record <b>(default)</b>
 *         <li><b>1</b> - Dynamic Record
 *     </ul><td>No
 * <tr><td><tt>layout</tt>
 *     <td>The access mode of the record.
 *     <ul>
 *         <li><b>0</b> - Direct Access <b>(default)</b>
 *         <li><b>1</b> - Hierarchical Access
 *         <li><b>2</b> - Direct access with inner classes
 *         <li><b>3</b> - Hierachical access with inner classes
 *     </ul><td>No
 * <tr><td><tt>style</tt>
 *     <td><ul>
 *         <li><b>0</b> - Notification Support
 *         <li><b>1</b> - No Notification Support <b>(default)</b>
 *     </ul><td>No
 * <tr><td><tt>namingScheme</tt>
 *     <td>Only used when layout = "2" or "3"
 *     <ul>
 *         <li><b>0</b> - Long names (for VAJ 2.0 compatibility)
 *         <li><b>1</b> - Short names <b>(default)</b>
 *     </ul><td>No
 * <tr><td><tt>j2ee</tt>
 *     <td>Generate J2EE style record (implements
 *         <code>javax.resource.cci.Streamable</code>).
 *         Default = "<b>no</b>".
 *     <td>No
 * <tr><td><tt>recAttrType</tt>
 *     <td>Record Attribute Type.  Default =
 *         <tt>com.ibm.ivj.eab.record.cobol.CobolRecordAttributes</tt>.
 *     <td>No
 * <tr><td><tt>genPrimArrTypes</tt>
 *     <td>Whether to generate accessors using primitive Java types
 *         or their corresponding object wrappers.  Default is
 *         <tt>false</tt>, to not generate primitive type accessors.
 *     <td>No
 * </table>
 * <p>
 * @author Paul Glezen
 */
public class RecordTask extends Task
{
   /** Default record attribute type - currently set to 
    *  "com.ibm.ivj.eab.record.cobol.CobolRecordAttributes"; */
   public static final String DEFAULT_REC_ATTR_TYPE = 
         "com.ibm.ivj.eab.record.cobol.CobolRecordAttributes";

   /** The name of the program that generates record source - currently
    *  set to <tt>com.ibm.ivj.eab.record.generator.codegen.Generator</tt>. */
   public static final String RECORD_GENERATOR =
                    "com.ibm.ivj.eab.record.generator.codegen.Generator";

   private Path classpath;
   private File targetDir;
   private String pkgName;
   private String className;
   private String recType;
   private String outputType   = "0";  // Custom Record
   private String layout       = "0";  // Direct Access
   private String style        = "1";  // No notification support
   private String recAttrType  = DEFAULT_REC_ATTR_TYPE;
   private String namingScheme = "1";  // Only used when layout = "2" or "3"
   private boolean j2ee        = true; // J2EE or CCF
   private boolean genPrimArrTypes = false; // true = int, false = Integer

   /**
    * Set the target directory in which the record package
    * heirarchy will be generated.
    */
   public void setTargetDir(File dir) {
      targetDir = dir; 
   }

   /**
    * Set the package name of the record to be generated.
    */
   public void setPkgName(String name) {
      pkgName = name; 
   }

   /**
    * Set the class name of the record to be generated.
    */
   public void setClassName(String name) {
      className = name; 
   }

   /**  
    * Set the fully-qualified class name of the record
    * type class used to generate the record class.
    */
   public void setRecType(String type) {
      recType = type; 
   }

   /**  
    * Set the output type for the record generation.
    * The input is expected to be either "0" or "1" representing
    * a custom record or dynamic record respectively.
    * <p>
    * @param type the record type
    */
   public void setOutputType(String type) {
      outputType = type; 
   }

   /**
    * Set the record layout for record generation.  The input
    * is expected to be one of the following:
    * <p><ul>
    * <li>"0" - Direct access
    * <li>"1" - Hierarchical access
    * <li>"2" - Direct access with inner classes
    * <li>"3" - Hierarchical access with inner classes
    * </ul><p>
    * @param layout the layout
    */
   public void setLayout(String layout) {
      this.layout = layout; 
   }

   /**
    * Determine whether notification support is incorporated
    * into the generated record.  The input is expected to be
    * either "0" or "1" indicating no notification support or
    * no notification support respectively.
    * <p>
    * @param style the style
    */
   public void setStyle(String style) {
      this.style = style; 
   }

   /**
    * Set the name of the record type attribute class used
    * by the generated record class.  It is initialized by
    * default to be {@link #DEFAULT_REC_ATTR_TYPE}.
    * <p>
    * @param type the attribute type
    */
   public void setRecAttrType(String type) {
      recAttrType = type; 
   }

   /**
    * Determine whether short or long names are to be used
    * for attributes.  The expected input is "0" for long
    * names; "1" for short names.
    * <p>
    * @param scheme the naming scheme
    */
   public void setNamingScheme(String scheme) {
      namingScheme = scheme; 
   }

   /**
    * Determine whether to generate J2EE style records or
    * CCF style records.  J2EE style records implement the
    * <code>javax.resource.cci.Streamable</code> interface.
    * <p>
    * @param j2ee use j2ee style records if set to true
    */
   public void setJ2ee(boolean j2ee) {
      this.j2ee = j2ee; 
   }

   /**
    * Generate accessors that return primitive data types.
    * Otherwise, wrappers such as <tt>Integer</tt> are
    * returned instead of <tt>int</tt>.
    * <p>
    * @param type accessor return type
    */
   public void setGenPrimArrTypes(boolean type) {
      genPrimArrTypes = type; 
   }

   /**
    * Set the classpath for finding dependent record type classes.
    * If there is already a classpath set internally, append
    * rather than replace the input to the existing path.
    * <p>
    * @param src - the path to add
    */
   public void setClasspath(Path src) {
      if (classpath == null) {
         classpath = src;
      }
      else {
         classpath.append(src);
      }
   }

   /**
    * Create a <code>Path</code> object to be populated with
    * path elements.
    * <p>
    * @return a sub-path of this object's <tt>Path</tt> instance.
    */
   public Path createClasspath() {
      if (classpath == null) {
         classpath = new Path(getProject());
      }
      return classpath.createPath();
   }

   /**
    * Return a reference to this object's classpath element.
    * <p>
    * @return the classpath
    */
   public Path getClasspath() {
      return classpath;
   }

   /**
    * Add a reference to another classpath element.
    * <p>
    * @param ref a reference to another <tt>Path</tt> instance
    */
   public void setClasspathRef(Reference ref) {
      createClasspath().setRefid(ref);
   }

   /**
    * Execute an EAB command {@link #RECORD_GENERATOR} to generate
    * a record type class.  The command is forked rather than run
    * in the same JVM since some EAB commands have <tt>exit</tt>
    * coded in their main method.
    */
   public void execute() throws BuildException
   {
      Vector argVec = new Vector();
      argVec.add("java");
      argVec.add("-classpath");
      argVec.add(classpath.concatSystemClasspath().toString());
      argVec.add(RECORD_GENERATOR);
      argVec.add(targetDir.getName());
      argVec.add(pkgName);
      argVec.add(className);
      argVec.add(recType);
      argVec.add(outputType);
      argVec.add(layout);
      argVec.add(style);
      argVec.add(recAttrType);
      if (namingScheme != null) {
         argVec.add(namingScheme);
      }
      if (j2ee) {
         argVec.add("GenCA");
      }
      if (genPrimArrTypes) {
         argVec.add("GenPrimArrTypes");
      }

      String[] args = new String[argVec.size()];
      for (int i=0; i < args.length; i++) {
         args[i] = (String) argVec.elementAt(i);
      }

      try {
         Execute exec = new Execute();
         exec.setCommandline(args);
         exec.setNewenvironment(true);
         int rc = exec.execute();
//         if ( rc != 0)
//           throw new BuildException("Record generation returned " + rc);
      } catch (java.io.IOException ioe) {
         throw new BuildException("Process Creation failure", ioe);
      }
   }
}
