/*
 * ============================================================================
 *                   The Apache Software License, Version 1.1
 * ============================================================================
 * 
 *    Copyright (C) 1999 The Apache Software Foundation. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modifica-
 * tion, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of  source code must  retain the above copyright  notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * 3. The end-user documentation included with the redistribution, if any, must
 *    include  the following  acknowledgment:  "This product includes  software
 *    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
 *    Alternately, this  acknowledgment may  appear in the software itself,  if
 *    and wherever such third-party acknowledgments normally appear.
 * 
 * 4. The names "Ant" and  "Apache Software Foundation"  must not be used to
 *    endorse  or promote  products derived  from this  software without  prior
 *    written permission. For written permission, please contact
 *    apache@apache.org.
 * 
 * 5. Products  derived from this software may not  be called "Apache", nor may
 *    "Apache" appear  in their name,  without prior written permission  of the
 *    Apache Software Foundation.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
 * APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
 * DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
 * ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
 * (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * This software  consists of voluntary contributions made  by many individuals
 * on behalf of the  Apache Software Foundation.  For more  information  on the 
 * Apache Software Foundation, please see <http://www.apache.org/>.
 */

package org.apache.tools.ant.taskdefs.optional.javarec;

import java.io.File;
import java.util.Vector;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.Execute;

/**
 * Generate a Cobol record type from a Cobol copy book.
 * This Ant task calls the <code>CobolImport</code> command to
 * create <code>RecordType</code> classes from a Cobol copy
 * book.  There are no nested tags for this task - only
 * attributes.
 * <h3>Parameters</h3>
 * <table border="1" cellpadding="2">
 * <tr><th>Attribute<th>Description<th>Required
 * <tr><td><tt>targetDir</tt>
 *     <td>The directory into which record type source will be
 *         generated.
 *     <td>Yes
 * <tr><td><tt>pkgName</tt>
 *     <td>The package in which to place the record type class.
 *     <td>Yes
 * <tr><td><tt>className</tt>
 *     <td>The name of the record type class.
 *     <td>Yes
 * <tr><td><tt>ccpfile</tt>
 *     <td>The name of the file containing the Cobol copy book.
 *     <td>Yes
 * <tr><td><tt>commarea</tt>
 *     <td>The names of the definition records for the COMMAREAs
 *         contained in the Cobol copy book file.  Each name in
 *         the string is separated by a colon.
 *         The default is <tt>DFHCOMMAREA</tt>.
 *     <td>No
 * <tr><td><tt>nocics</tt>
 *     <td>If <tt>nocics</tt> is true, the <tt>commarea</tt> is not
 *         required.  Otherwise it is assumed that the record type
 *         is intended for a CICS transaction, which is the default
 *         behavior.
 *     <td>No
 * <tr><td><tt>genold</tt>
 *     <td>If set to <tt>true</tt>, use a Java <tt>double</tt> to represent
 *         decimals.  Otherwise use Java <tt>java.math.BigDecimal</tt>.
 *         The default is <tt>false</tt>.
 *     <td>No
 * </table><p>
 * <h3>Examples</h3>
 * In the following example, the copy book is stored in a file
 * named <tt>CustSearch.ccp</tt>.  The record type class will be
 * named <tt>CustSearchRecordType</tt> and be placed in the file
 * <tt>src/com/yourfirm/record/CustSearchRecordType</tt>.
 * <p><pre>
 * &lt;mkdir dir="src/com/yourfirm/record"/&gt;
 * &lt;generated-record-type targetDir="src"
 *                           pkgName="com.yourfirm.record"
 *                           className="CustSearchRecordType"
 *                           ccpfile="CustSearch.ccp"/&gt;
 * </pre>
 * The creation of the directory for <tt>CustSearchRecordType</tt>
 * before hand was necessary for the EAB command line tool to
 * completel successfully.
 * <p>
 * @author Paul Glezen
 */
public class RecordTypeTask extends Task {
    /** Default copy book section - currently set to <tt>DFHCOMMAREA</tt>. */
    public static final String DEFAULT_COMMAREA = "DFHCOMMAREA";

    /** The fully qualified class name of the record type generating
     *  currently set to 
     *  <tt>com.ibm.ivj.eab.record.importer.gentype.cobol.CobolImport</tt>.
     */
    public static final String RECORD_TYPE_GENERATOR = 
        "com.ibm.ivj.eab.record.importer.gentype.cobol.CobolImport";

    private File    targetDir;
    private File    ccpfile;
    private String  pkgName;
    private String  className;
    private String  commarea = "DFHCOMMAREA";
    private boolean   nocics = false;
    private boolean   genold = false;

    /** Set the target directory of the generated source file's
     *  package hierarchy.  */
    public void setTargetDir(File dir) { 
        targetDir = dir; 
    }

    /** Set the file containing the Cobol copy book source.  */
    public void setCcpfile(File file) { 
        ccpfile = file; 
    }

    /** Set the package for the generated record type class. */
    public void setPkgName(String name) { 
        pkgName = name; 
    }

    /** Set the class name for the generated record type.  */
    public void setClassName(String name) { 
        className = name; 
    }

    /** Set the COMMAREA section of the copy book to be used for
      * for record type generation.  If it is necessary to specify
      * a hierarchy of names, these should be separated by colons. */
    public void setCommarea(String name) { 
        commarea = name; 
    }

	 /**  
	  * Set this property to <tt>true</tt> when your EIS is not a CICS
	  * system (IMS, for example).  The default is <tt>true</tt> (CICS
	  * is assumed.)
	  */
    public void setNocics(boolean nocics) { 
        this.nocics = nocics; 
    }

	 /**
	  * If set to <tt>true</tt>, a Java <tt>double</tt> is used to 
	  * represent decimals.  Otherwise <tt>java.math.BigDecimal</tt>
	  * is used.  The default is false.
	  */
    public void setGenold(boolean genold) { 
        this.genold = genold; 
    }

	/**
	 * Check that the required parameters are present.
	 * <p>
	 * @throws org.apache.tools.ant.BuildException if one of the required
	 *             parameters if found to be missing.
	 */
	protected void checkRequiredParams() throws BuildException {
		String msg = "";
		if (pkgName == null  ||  pkgName.equals("")) {
			msg += "Non-default package must be specified for record type.\n";
		}
		if (className == null  ||  className.equals("")) {
			msg += "Record Type className missing.\n";
		}
		if (ccpfile == null) {
			msg += "Cobol copy book entry missing.\n";
		}
		if (!msg.equals(""))
			throw new BuildException(msg);
	}

	 /**
	  * Perform the invocation of the EAB command to generate the record
	  * type from a Cobol copy book.
	  * <p>
	  * @throws org.apache.tools.ant.BuildException if a non-zero return code
	  *             results or an java.io.IOException is thrown (because of
	  *             process creation failure).
	  */
    public void execute() throws BuildException {
		checkRequiredParams();
        Vector argVec = new Vector();
        argVec.add("java");
        argVec.add("-classpath");
        argVec.add(System.getProperty("java.class.path", "."));
        argVec.add(RECORD_TYPE_GENERATOR);
        argVec.add(targetDir.getAbsolutePath());
        argVec.add(pkgName);
        argVec.add(className);
        argVec.add(commarea);
        argVec.add(ccpfile.getAbsolutePath());
        if (nocics) {
            argVec.add("-nocics");
        }
        if (genold) {
            argVec.add("-genold");
        }

        String[] args = new String[argVec.size()];
        for (int i=0; i < args.length; i++) {
            args[i] = (String) argVec.elementAt(i);
        }

        try {
            Execute exec = new Execute();
            exec.setCommandline(args);
            exec.setNewenvironment(true);
            int rc = exec.execute();
            if (rc != 0) {
                throw new BuildException("Record type generation returned " + rc);
            }
        } catch (java.io.IOException ioe) {
            throw new BuildException("Process Creation failure", ioe);
        }
    }
}
